@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -12,24 +13,21 @@
 """
 Contains PersistentManagerBase, PersistentExecutionManagerBase classes and subclasses,
 as well as some convenience functions.
-
-:authors: M. Marin (KTH)
-:date:   12.10.2020
 """
-import uuid
+from dataclasses import dataclass
+from multiprocessing import Lock, Process
+import os
+from queue import Empty, Queue
+import signal
 import socket
 import socketserver
+from subprocess import PIPE, Popen
 import sys
-import os
-import signal
 import threading
 import time
-from dataclasses import dataclass
-from subprocess import Popen, PIPE
-from multiprocessing import Process, Lock
-from queue import Queue, Empty
-from ..utils.helpers import Singleton
+import uuid
 from ..utils.execution_resources import persistent_process_semaphore
+from ..utils.helpers import Singleton
 from .execution_manager_base import ExecutionManagerBase
 
 if sys.platform == "win32":
@@ -147,15 +145,21 @@ class PersistentManagerBase:
 
     def _log_stdout(self):
         """Puts stdout from the process into the queue (it will be consumed by issue_command())."""
-        for line in iter(self._persistent.stdout.readline, b''):
-            data = line.decode("UTF8", "replace").rstrip()
-            self._msg_queue.put(dict(type="stdout", data=data))
+        try:
+            for line in iter(self._persistent.stdout.readline, b""):
+                data = line.decode("UTF8", "replace").rstrip()
+                self._msg_queue.put(dict(type="stdout", data=data))
+        except ValueError:
+            pass
 
     def _log_stderr(self):
         """Puts stderr from the process into the queue (it will be consumed by issue_command())."""
-        for line in iter(self._persistent.stderr.readline, b''):
-            data = line.decode("UTF8", "replace").rstrip()
-            self._msg_queue.put(dict(type="stderr", data=data))
+        try:
+            for line in iter(self._persistent.stderr.readline, b""):
+                data = line.decode("UTF8", "replace").rstrip()
+                self._msg_queue.put(dict(type="stderr", data=data))
+        except ValueError:
+            pass
 
     def make_complete_command(self, cmd):
         lines = cmd.splitlines()
@@ -332,8 +336,8 @@ class PersistentManagerBase:
         """
         if not self.is_persistent_alive():
             raise PersistentIsDead()
-        req_args_sep = '\u001f'  # Unit separator
-        args_sep = '\u0091'  # Private Use 1
+        req_args_sep = "\u001f"  # Unit separator
+        args_sep = "\u0091"  # Private Use 1
         args = args_sep.join(args)
         msg = f"{request}{req_args_sep}{args}"
         with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
@@ -403,19 +407,32 @@ class PersistentManagerBase:
 
     def interrupt_persistent(self):
         """Interrupts the persistent process."""
-        threading.Thread(target=self._do_interrupt_persistent).start()
+        queue = Queue()
+        thread = threading.Thread(target=self._do_interrupt_persistent, args=(queue,))
+        thread.start()
+        # Wait till the process becomes idle, then put None to the queue
         self._wait()
+        queue.put(None)
+        thread.join()
 
-    def _do_interrupt_persistent(self):
+    def _do_interrupt_persistent(self, queue):
         persistent = self._persistent  # Make local copy; other threads may set self._persistent to None while sleeping.
         if persistent is None:
             return
-        if sys.platform == "win32":
-            p = Process(target=_send_ctrl_c, args=(persistent.pid,))
-            p.start()
-            p.join()
-        else:
-            persistent.send_signal(signal.SIGINT)
+        while True:
+            if sys.platform == "win32":
+                p = Process(target=_send_ctrl_c, args=(persistent.pid,))
+                p.start()
+                p.join()
+            else:
+                persistent.send_signal(signal.SIGINT)
+            try:
+                # Wait 2 seconds to get an item from the queue, which means the process has become idle...
+                # Otherwise just keep interrupting it
+                queue.get(timeout=2)
+                break
+            except Empty:
+                pass
         self.set_running_until_completion(False)
 
     def is_persistent_alive(self):
@@ -528,6 +545,7 @@ class _PersistentManagerFactory(metaclass=Singleton):
     persistent_managers = {}
     """Maps keys to associated PersistentManagerBase instances."""
     _factory_open = OpenSign()
+    _idle_manager_lock = threading.Lock()
 
     @staticmethod
     def _emit_persistent_started(logger, key, language):
@@ -555,7 +573,7 @@ class _PersistentManagerFactory(metaclass=Singleton):
             constructor (Callable): the persistent manager constructor
             logger (LoggerInterface)
             args (list): the arguments to launch the persistent process
-            group_id (str): item group that will execute using this persistent
+            group_id (str): Item group for sharing this persistent process
 
         Returns:
             PersistentManagerBase: persistent manager or None if factory has been closed
@@ -563,16 +581,17 @@ class _PersistentManagerFactory(metaclass=Singleton):
         while not persistent_process_semaphore.acquire(timeout=0.5):
             if not self._factory_open:
                 return None
-            idle_pms = self._get_idle_persistent_managers()
-            # Try to reuse
-            pm = self._reuse_persistent_manager(idle_pms, logger, args, group_id)
-            if pm:
-                return pm
-            # Kill an idle pm if any
-            if idle_pms:
-                key, pm = idle_pms[0]
-                pm.kill_process()
-                del self.persistent_managers[key]
+            with self._idle_manager_lock:
+                idle_pms = self._get_idle_persistent_managers()
+                # Try to reuse
+                pm = self._reuse_persistent_manager(idle_pms, logger, args, group_id)
+                if pm:
+                    return pm
+                # Kill an idle pm if any
+                if idle_pms:
+                    key, pm = idle_pms[0]
+                    pm.kill_process()
+                    del self.persistent_managers[key]
         # We got permission to create a new process
         # Try to reuse one last time just in case things changed quickly
         idle_pms = self._get_idle_persistent_managers()
@@ -597,12 +616,13 @@ class _PersistentManagerFactory(metaclass=Singleton):
 
         Args:
             key (tuple): persistent identifier
-        Returns:
-            generator: stdout and stderr messages (dictionaries with two keys: type, and data)
+
+        Yields:
+            dict: stdout and stderr messages (dictionaries with two keys: type, and data)
         """
         pm = self.persistent_managers.get(key)
         if pm is None:
-            return ()
+            return
         yield from pm.restart_persistent()
 
     def interrupt_persistent(self, key):
@@ -634,14 +654,13 @@ class _PersistentManagerFactory(metaclass=Singleton):
             key (tuple): persistent identifier
             cmd (str): command to issue
 
-        Returns:
-            generator: stdin, stdout, and stderr messages (dictionaries with two keys: type, and data)
+        Yields:
+            dict: stdin, stdout, and stderr messages (dictionaries with two keys: type, and data)
         """
         pm = self.persistent_managers.get(key)
         if pm is None:
-            return ()
-        for msg in pm.issue_command(cmd, add_history=True, catch_exception=False):
-            yield msg
+            return
+        yield from pm.issue_command(cmd, add_history=True, catch_exception=False)
 
     def is_persistent_command_complete(self, key, cmd):
         """Checks whether a command is complete.
@@ -684,8 +703,6 @@ class _PersistentManagerFactory(metaclass=Singleton):
         """
         pm = self.persistent_managers.get(key)
         if pm is None:
-            return
-        if sys.platform == "win32":
             return ""
         return pm.get_history_item(text, prefix, backwards)
 
@@ -772,12 +789,12 @@ class PersistentExecutionManagerBase(ExecutionManagerBase):
     def __init__(self, logger, args, commands, alias, kill_completed_processes, group_id=None):
         """
         Args:
-            logger (LoggerInterface): a logger instance
-            args (list): List of args to start the persistent process
-            commands (list): List of commands to execute in the persistent process
-            alias (str): an alias name for the manager
-            kill_completed_processes (bool): if True, the persistent process will be killed after execution
-            group_id (str, optional): item group that will execute using this kernel
+            logger (LoggerInterface): Logger instance
+            args (list): Program and cmd line args for the persistent process
+            commands (list): List of commands to execute in the persistent process (including the script cmd line args)
+            alias (str): Alias name for the manager
+            kill_completed_processes (bool): If True, the persistent process will be killed after execution
+            group_id (str, optional): Item group for sharing this persistent process
         """
         super().__init__(logger)
         self._args = args
