@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -9,28 +10,24 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
 
-"""
-Contains the ProcessExecutionManager class.
+""" Contains the ProcessExecutionManager class. """
 
-:authors: M. Marin (KTH)
-:date:   12.10.2020
-"""
-
-import sys
 import subprocess
+import sys
 from threading import Thread
-from .execution_manager_base import ExecutionManagerBase
 from ..utils.execution_resources import one_shot_process_semaphore
+from .execution_manager_base import ExecutionManagerBase
 
 
 class ProcessExecutionManager(ExecutionManagerBase):
-    def __init__(self, logger, program, *args, workdir=None):
+    def __init__(self, logger, program, args, workdir=None):
         """Class constructor.
 
         Args:
-            logger (LoggerInterface): a logger instance
-            program (str): Path to program to run in the subprocess (e.g. julia.exe)
-            *args: List of arguments for the program (e.g. path to script file)
+            logger (LoggerInterface): Logger instance
+            program (str): Path to program to run in the subprocess (e.g. 'path/to/julia')
+            args (list): List of arguments for the program (e.g. path to script file)
+            workdir (str): Work directory for the subprocess.Popen process
         """
         super().__init__(logger)
         self._process = None
@@ -47,7 +44,7 @@ class ProcessExecutionManager(ExecutionManagerBase):
                 return 0
             try:
                 self._process = subprocess.Popen(
-                    [self._program, *self._args],
+                    [self._program] + self._args,
                     stdout=subprocess.PIPE,
                     stderr=subprocess.PIPE,
                     cwd=self._workdir,
@@ -59,6 +56,8 @@ class ProcessExecutionManager(ExecutionManagerBase):
                 return 1
             msg = dict(type="execution_started", program=self._program, args=" ".join(self._args))
             self._logger.msg_standard_execution.emit(msg)
+            running = "# Running" + " ".join([self._program] + self._args)
+            self._logger.msg_standard_execution.emit({"type": "stdin", "data": running})
             Thread(target=self._log_stdout, args=(self._process.stdout,), daemon=True).start()
             Thread(target=self._log_stderr, args=(self._process.stderr,), daemon=True).start()
             return self._process.wait()
@@ -69,11 +68,15 @@ class ProcessExecutionManager(ExecutionManagerBase):
             self._process.terminate()
 
     def _log_stdout(self, stdout):
-        for line in iter(stdout.readline, b''):
-            self._logger.msg_proc.emit(line.decode("UTF8", "replace").strip())
+        for line in iter(stdout.readline, b""):
+            line = line.decode("UTF8", "replace").strip()
+            self._logger.msg_proc.emit(line)
+            self._logger.msg_standard_execution.emit({"type": "stdout", "data": line})
         stdout.close()
 
     def _log_stderr(self, stderr):
-        for line in iter(stderr.readline, b''):
-            self._logger.msg_proc_error.emit(line.decode("UTF8", "replace").strip())
+        for line in iter(stderr.readline, b""):
+            line = line.decode("UTF8", "replace").strip()
+            self._logger.msg_proc_error.emit(line)
+            self._logger.msg_standard_execution.emit({"type": "stderr", "data": line})
         stderr.close()
