@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -14,13 +15,11 @@ The function of this class can be used for generation of keys for enabling secur
 of the remote spine_server. The code has been copied based on an example at (by Chris Laws):
 https://github.com/zeromq/pyzmq/blob/main/examples/security/generate_certificates.py
 
-:authors: P. Pääkkönen (VTT)
-:date:   15.09.2021
 """
 
 import os
-import sys
 import shutil
+import sys
 import zmq.auth
 
 
