@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -11,27 +12,24 @@
 
 """
 Contains EngineServer class for running a Zero-MQ server with Spine Engine.
-:authors: P. Pääkkönen (VTT), P. Savolainen (VTT)
-:date:   19.08.2021
 """
+import enum
+import ipaddress
 import json.decoder
 import os
 import queue
-import time
 import threading
-import ipaddress
-import enum
 import uuid
 import zmq
 from zmq.auth.thread import ThreadAuthenticator
-from spine_engine.server.util.server_message import ServerMessage
-from spine_engine.server.request import Request
-from spine_engine.server.remote_execution_service import RemoteExecutionService
+from spine_engine.server.persistent_execution_service import PersistentExecutionService
 from spine_engine.server.ping_service import PingService
 from spine_engine.server.project_extractor_service import ProjectExtractorService
-from spine_engine.server.project_retriever_service import ProjectRetrieverService
-from spine_engine.server.persistent_execution_service import PersistentExecutionService
 from spine_engine.server.project_remover_service import ProjectRemoverService
+from spine_engine.server.project_retriever_service import ProjectRetrieverService
+from spine_engine.server.remote_execution_service import RemoteExecutionService
+from spine_engine.server.request import Request
+from spine_engine.server.util.server_message import ServerMessage
 
 
 class ServerSecurityModel(enum.Enum):
@@ -58,9 +56,9 @@ class EngineServer(threading.Thread):
             if not sec_folder:
                 raise ValueError("Path to security folder missing")
             base_dir = sec_folder
-            self.keys_dir = os.path.join(base_dir, 'certificates')
-            self.public_keys_dir = os.path.join(base_dir, 'public_keys')
-            self.secret_keys_dir = os.path.join(base_dir, 'private_keys')
+            self.keys_dir = os.path.join(base_dir, "certificates")
+            self.public_keys_dir = os.path.join(base_dir, "public_keys")
+            self.secret_keys_dir = os.path.join(base_dir, "private_keys")
             if not os.path.exists(self.keys_dir):
                 raise ValueError(f"Security folder: {self.keys_dir} does not exist")
             if not os.path.exists(self.public_keys_dir):
@@ -84,7 +82,8 @@ class EngineServer(threading.Thread):
         """Closes the server by sending a KILL message to this thread using a PAIR socket."""
         if self.auth is not None:
             self.auth.stop()
-            time.sleep(0.2)  # wait a bit until authenticator has been closed
+            while self.auth.is_alive():
+                pass
         self.ctrl_msg_sender.send(b"KILL")
         self.join()  # Wait for the thread to finish and sockets to close
         self.ctrl_msg_sender.close()  # Close this in the same thread that it was created in
@@ -144,7 +143,12 @@ class EngineServer(threading.Thread):
                             self.send_init_failed_reply(frontend, request.connection_id(), msg)
                             continue
                         worker = RemoteExecutionService(
-                            self._context, request, job_id, project_dir, persistent_exec_mngr_q
+                            self._context,
+                            request,
+                            job_id,
+                            project_dir,
+                            persistent_exec_mngr_q,
+                            self.port,
                         )
                     elif request.cmd() == "stop_execution":
                         worker = workers.get(request.request_id(), None)  # Get DAG execution worker based on job Id
@@ -156,6 +160,16 @@ class EngineServer(threading.Thread):
                         worker.stop_engine()
                         request.send_response(frontend, ("server_status_msg", "DAG worker stopped"))
                         continue
+                    elif request.cmd() == "answer_prompt":
+                        worker = workers.get(request.request_id(), None)  # Get DAG execution worker based on job Id
+                        if not worker:
+                            print(f"Worker for job_id:{request.request_id()} not found")
+                            msg = f"Answering prompt failed. Worker for job_id:{request.request_id()} not found."
+                            self.send_init_failed_reply(frontend, request.connection_id(), msg)
+                            continue
+                        prompter_id, answer = request.data()
+                        worker.answer_prompt(prompter_id, answer)
+                        continue
                     elif request.cmd() == "retrieve_project":
                         project_dir = project_dirs.get(request.request_id(), None)  # Get project dir based on job_id
                         if not project_dir:
@@ -178,7 +192,7 @@ class EngineServer(threading.Thread):
                             )
                             self.send_init_failed_reply(frontend, request.connection_id(), msg)
                             continue
-                        worker = PersistentExecutionService(self._context, request, job_id, exec_mngr)
+                        worker = PersistentExecutionService(self._context, request, job_id, exec_mngr, self.port)
                     elif request.cmd() == "remove_project":
                         project_dir = project_dirs.get(request.request_id(), None)  # Get project dir based on job_id
                         if not project_dir:
@@ -253,9 +267,7 @@ class EngineServer(threading.Thread):
         b_json_str_server_msg = msg[1]  # binary string
         if len(b_json_str_server_msg) <= 10:  # Message size too small
             print(f"User data frame too small [{len(msg[1])}]. msg:{msg}")
-            self.send_init_failed_reply(
-                socket, msg[0], f"User data frame too small " f"- Malformed message sent to server."
-            )
+            self.send_init_failed_reply(socket, msg[0], "User data frame too small. Malformed message sent to server.")
             return None
         try:
             json_str_server_msg = b_json_str_server_msg.decode("utf-8")  # json string
@@ -331,7 +343,7 @@ class EngineServer(threading.Thread):
         allowed_str = "\n".join(allowed)
         print(f"StoneHouse security activated. Allowed endpoints ({len(allowed)}):\n{allowed_str}")
         # Tell the authenticator how to handle CURVE requests
-        auth.configure_curve(domain='*', location=zmq.auth.CURVE_ALLOW_ANY)
+        auth.configure_curve(domain="*", location=zmq.auth.CURVE_ALLOW_ANY)
         server_secret_file = os.path.join(self.secret_keys_dir, "server.key_secret")
         server_public, server_secret = zmq.auth.load_certificate(server_secret_file)
         frontend.curve_secretkey = server_secret
