@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -11,12 +12,10 @@
 
 """
 Contains a class for remote persistent execution manager related actions.
-:authors: P. Savolainen (VTT)
-:date:   26.9.2022
 """
 
-import threading
 import json
+import threading
 import zmq
 from spine_engine.server.service_base import ServiceBase
 
@@ -24,23 +23,29 @@ from spine_engine.server.service_base import ServiceBase
 class PersistentExecutionService(threading.Thread, ServiceBase):
     """Class for interacting with a persistent execution manager running on server."""
 
-    def __init__(self, context, request, job_id, persistent_exec_mngr):
+    def __init__(self, context, request, job_id, persistent_exec_mngr, frontend_port):
         """
         Args:
             context (zmq.Context): Context for this handler.
             request (Request): Client request
             job_id (str): Worker thread Id
             persistent_exec_mngr (PersistentExecutionManagerBase): Persistent execution manager
+            frontend_port (int): Server frontend port
         """
         super(PersistentExecutionService, self).__init__(name="PersistentExecutionService")
         ServiceBase.__init__(self, context, request, job_id)
         self.persistent_exec_mngr = persistent_exec_mngr
         self.push_socket = self.context.socket(zmq.PUSH)
+        self.frontend_port = frontend_port
 
     def run(self):
         """Executes client's command in execution service and returns the response back to client."""
         self.worker_socket.connect("inproc://backend")
-        pub_port = self.push_socket.bind_to_random_port("tcp://*")
+        pub_port = self.push_socket.bind_to_random_port(
+            "tcp://*",
+            min_port=self.frontend_port,
+            max_port=self.frontend_port + self.n_port_range,
+        )
         pm = self.persistent_exec_mngr._persistent_manager
         cmd_type = self.request.data()[1]  # Command type for persistent manager, e.g. 'is_complete'
         cmd = self.request.data()[2]  # Command to process in persistent manager
@@ -52,7 +57,7 @@ class PersistentExecutionService(threading.Thread, ServiceBase):
             for msg in pm.issue_command(cmd, add_history=True, catch_exception=False):
                 json_msg = json.dumps(msg)
                 self.push_socket.send(json_msg.encode("utf-8"))  # This blocks until somebody is pulling (receiving)
-            self.push_socket.send(b'END')
+            self.push_socket.send(b"END")
             retval_tuple = cmd_type, "ok"
         elif cmd_type == "get_completions":
             retval = pm.get_completions(cmd)
@@ -66,7 +71,7 @@ class PersistentExecutionService(threading.Thread, ServiceBase):
             for msg in pm.restart_persistent():
                 json_msg = json.dumps(msg)
                 self.push_socket.send(json_msg.encode("utf-8"))
-            self.push_socket.send(b'END')
+            self.push_socket.send(b"END")
             retval_tuple = cmd_type, "ok"
         elif cmd_type == "interrupt_persistent":
             pm.interrupt_persistent()
