@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -8,27 +9,22 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-
-"""
-Contains RemoteExecutionService class that executes a single DAG on the Spine Engine Server.
-:authors: P. Pääkkönen (VTT), P. Savolainen (VTT)
-:date:   24.08.2021
-"""
+""" Contains RemoteExecutionService class that executes a single DAG on the Spine Engine Server. """
 
 import os
 import threading
 import zmq
 from spine_engine import SpineEngine
-from spine_engine.utils.helpers import get_file_size
 from spine_engine.server.service_base import ServiceBase
 from spine_engine.server.util.event_data_converter import EventDataConverter
+from spine_engine.utils.helpers import get_file_size
 
 
 class RemoteExecutionService(threading.Thread, ServiceBase):
     """Executes a DAG contained in the client request. Project must
     be on server before running this service."""
 
-    def __init__(self, context, request, job_id, project_dir, persistent_exec_mngr_q):
+    def __init__(self, context, request, job_id, project_dir, persistent_exec_mngr_q, frontend_port):
         """
         Args:
             context (zmq.Context): Context for this handler.
@@ -36,6 +32,7 @@ class RemoteExecutionService(threading.Thread, ServiceBase):
             job_id (str): Worker thread Id
             project_dir (str): Absolute path to a server directory where the project has been extracted to
             persistent_exec_mngr_q (queue.Queue): Queue for storing persistent exec. managers (consumed in frontend)
+            frontend_port (int): Server frontend port number
         """
         super().__init__(name="RemoteExecutionServiceThread")
         ServiceBase.__init__(self, context, request, job_id)
@@ -45,6 +42,7 @@ class RemoteExecutionService(threading.Thread, ServiceBase):
         self.persistent_keys = dict()  # Mapping of item_name to a persistent execution manager key
         self.persistent_exec_mngrs = dict()  # Mapping of per. execution manager key to per. execution manager
         self.persist_q = persistent_exec_mngr_q
+        self.frontend_port = frontend_port
         self.items = list()
 
     def collect_persistent_keys(self, event_type, data):
@@ -86,7 +84,11 @@ class RemoteExecutionService(threading.Thread, ServiceBase):
         """Sends an execution started response to start execution request. Runs Spine Engine
         and sends the events to the client using a publish socket."""
         self.worker_socket.connect("inproc://backend")
-        push_port = self.push_socket.bind_to_random_port("tcp://*")
+        # Bind to specific port range, so we know which ports to open for containers
+        # min_port is inclusive, max_port is exclusive
+        push_port = self.push_socket.bind_to_random_port(
+            "tcp://*", min_port=self.frontend_port, max_port=self.frontend_port + self.n_port_range
+        )
         engine_data = self.request.data()
         print(f"Executing DAG [{self.job_id}] ...")
         # Send reply to 'start_execution' request to client with the push socket port for
@@ -104,7 +106,7 @@ class RemoteExecutionService(threading.Thread, ServiceBase):
                 self.collect_running_items(self.engine._running_items)
                 json_event = EventDataConverter.convert(event_type, data)
                 self.push_socket.send_multipart([json_event.encode("utf-8")])  # Blocks until the client pulls
-                if data == "COMPLETED" or data == "FAILED" or data == "USER_STOPPED":
+                if event_type == "dag_exec_finished":
                     break
         except StopIteration:
             # Raised by SpineEngine._get_event_stream() generator if we try to get_event() after
@@ -120,8 +122,6 @@ class RemoteExecutionService(threading.Thread, ServiceBase):
             self.push_socket.send_multipart([json_error_event.encode("utf-8")])
             self.send_completed()
             return
-        finally:
-            self.engine.wait()
         if data != "USER_STOPPED":
             resources = self.collect_resources()
             self.persist_q.put(self.persistent_exec_mngrs)  # Put new persistent execution managers to queue
@@ -157,6 +157,10 @@ class RemoteExecutionService(threading.Thread, ServiceBase):
         """Stops DAG execution."""
         self.engine.stop()
 
+    def answer_prompt(self, prompter_id, answer):
+        """Answers prompt."""
+        self.engine.answer_prompt(prompter_id, answer)
+
     def close(self):
         """Cleans up sockets after worker is finished."""
         super().close()
@@ -176,15 +180,12 @@ class RemoteExecutionService(threading.Thread, ServiceBase):
         # Adjust project_dir to point to the local folder
         remote_folder = input_data["project_dir"]  # Project directory on client
         input_data["project_dir"] = local_project_dir  # Project directory on server
-        # Loop specs
-        specs_keys = input_data["specifications"].keys()
-        for specs_key in specs_keys:
+        for specs_key, spec_item in input_data["specifications"].items():
             spec_item = input_data["specifications"][specs_key]
-            i = 0
-            for specItemInfo in spec_item:
+            for i, item_info in enumerate(spec_item):
                 # Adjust definition_file_path in specs to point to the server folder
-                if "definition_file_path" in specItemInfo:
-                    original_def_file_path = specItemInfo["definition_file_path"]  # Absolute path on client machine
+                if "definition_file_path" in item_info:
+                    original_def_file_path = item_info["definition_file_path"]  # Absolute path on client machine
                     # Make sure path separators match the OS separator
                     original_def_file_path = original_def_file_path.replace("\\", os.path.sep)
                     # Remove part of definition file path that references client machine path to get
@@ -193,25 +194,26 @@ class RemoteExecutionService(threading.Thread, ServiceBase):
                     rel_def_file_path = original_def_file_path.replace(remote_folder + "/", "")
                     modified = os.path.join(local_project_dir, rel_def_file_path)  # Absolute path on server machine
                     input_data["specifications"][specs_key][i]["definition_file_path"] = modified
-                i += 1
-                # Modify Python executable path and kernel spec because those refer to paths on client's machine
-                if "execution_settings" in specItemInfo and specItemInfo["tooltype"] == "python":
-                    if specItemInfo["execution_settings"]["use_jupyter_console"]:
+                # Modify Python and Julia execution settings to refer to server paths
+                if "execution_settings" in item_info and item_info["tooltype"] == "python":
+                    if item_info["execution_settings"].get("use_jupyter_console", False):
                         # Replace kernel_spec_name with the default kernel spec 'python3' (must be available on server)
-                        specItemInfo["execution_settings"]["kernel_spec_name"] = "python3"
+                        item_info["execution_settings"]["kernel_spec_name"] = "python3"
                     else:
-                        # Replace Python executable exec with "" because client's Python is not available on server
-                        specItemInfo["execution_settings"]["executable"] = ""
+                        # Replace Python executable with "" because client's Python is not available on server
+                        item_info["execution_settings"]["executable"] = ""
+                if "execution_settings" in item_info and item_info["tooltype"] == "julia":
+                    if item_info["execution_settings"].get("use_jupyter_console", False):
+                        # Replace kernel with the default kernel 'julia-1.11' (must be available on server)
+                        item_info["execution_settings"]["kernel_spec_name"] = "julia-1.11"
+                    else:
+                        # Replace Julia executable with "" and Julia project
+                        item_info["execution_settings"]["executable"] = "julia"
+                        item_info["execution_settings"]["project"] = ""
         # Loop items
         items_keys = input_data["items"].keys()
         for items_key in items_keys:
             # Force execute in source dir
             if "execute_in_work" in input_data["items"][items_key]:
                 input_data["items"][items_key]["execute_in_work"] = False
-        # Edit app settings dictionary
-        # Replace Julia path and Julia project path with an empty string so that the server uses the Julia in PATH
-        input_data["settings"]["appSettings/juliaPath"] = ""
-        input_data["settings"]["appSettings/juliaProjectPath"] = ""
-        # Replace Julia kernel
-        input_data["settings"]["appSettings/juliaKernel"] = "julia-1.8"  # (must be available on server)
         return input_data
