@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -11,8 +12,6 @@
 
 """
 Contains a class for client requests received at server.
-:author: P. Pääkkönen (VTT), P. Savolainen (VTT)
-:date:   19.8.2021
 """
 
 import json
