@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
@@ -12,6 +13,4 @@
 """
 This package contains utility classes of remote server part of the Spine Engine.
 
-:authors: P. Pääkkönen (VTT), P. Savolainen (VTT)
-:date:    23.8.2021
 """
