@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -8,58 +9,48 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-
-"""
-Contains the SpineEngine class for running Spine Toolbox DAGs.
-
-:authors: M. Marin (KTH)
-:date:   20.11.2019
-"""
-
+""" Contains the SpineEngine class for running Spine Toolbox DAGs. """
 from enum import Enum, unique
+from itertools import product
+import multiprocessing as mp
 import os
 import threading
-import multiprocessing as mp
-from itertools import product
 import networkx as nx
-from dagster import (
-    PipelineDefinition,
-    SolidDefinition,
-    InputDefinition,
-    OutputDefinition,
-    DependencyDefinition,
-    ModeDefinition,
-    Output,
-    Failure,
-    DagsterEventType,
-    default_executors,
-    AssetMaterialization,
-    execute_pipeline_iterator,
-)
 from spinedb_api import append_filter_config, name_from_dict
-from spinedb_api.spine_db_server import db_server_manager
-from spinedb_api.filters.tools import filter_config
-from spinedb_api.filters.scenario_filter import scenario_name_from_dict
 from spinedb_api.filters.execution_filter import execution_filter_config
+from spinedb_api.filters.scenario_filter import scenario_name_from_dict
+from spinedb_api.filters.tools import filter_config
+from spinedb_api.spine_db_server import db_server_manager
 from .exception import EngineInitFailed
 from .execution_managers.persistent_execution_manager import (
     disable_persistent_process_creation,
     enable_persistent_process_creation,
 )
-from .utils.helpers import (
-    AppSettings,
-    inverted,
-    create_timestamp,
-    make_dag,
-    ExecutionDirection as ED,
-    ItemExecutionFinishState,
+from .jumpster import (
+    Failure,
+    Finalization,
+    InputDefinition,
+    JumpsterEventType,
+    Output,
+    PipelineDefinition,
+    SolidDefinition,
+    execute_pipeline_iterator,
 )
-from .utils.execution_resources import one_shot_process_semaphore, persistent_process_semaphore
-from .utils.queue_logger import QueueLogger
-from .project_item_loader import ProjectItemLoader
-from .multithread_executor.executor import multithread_executor
 from .project_item.connection import Connection, Jump
-from .shared_memory_io_manager import shared_memory_io_manager
+from .project_item_loader import ProjectItemLoader
+from .utils.execution_resources import one_shot_process_semaphore, persistent_process_semaphore
+from .utils.helpers import (
+    ItemExecutionFinishState,
+    create_timestamp,
+    dag_edges,
+    inverted,
+    make_connections,
+    make_dag,
+    required_items_for_execution,
+)
+from .utils.helpers import AppSettings
+from .utils.helpers import ExecutionDirection as ED
+from .utils.queue_logger import QueueLogger
 
 
 @unique
@@ -75,12 +66,6 @@ class SpineEngineState(Enum):
         return str(self.name)
 
 
-class _JumpPipelineDefinition(PipelineDefinition):
-    def __init__(self, *args, jumps=None, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.jumps = jumps if jumps is not None else []
-
-
 class SpineEngine:
     """
     An engine for executing a Spine Toolbox DAG-workflow.
@@ -102,7 +87,7 @@ class SpineEngine:
     ):
         """
         Args:
-            items (list(dict)): List of executable item dicts.
+            items (dict): A mapping from item name to item dict
             specifications (dict(str,list(dict))): A mapping from item type to list of specification dicts.
             connections (list of dict): List of connection dicts
             jumps (list of dict, optional): List of jump dicts
@@ -110,62 +95,56 @@ class SpineEngine:
             settings (dict): Toolbox execution settings.
             project_dir (str): Path to project directory.
             execution_permits (dict(str,bool)): A mapping from item name to a boolean value, False indicating that
-                the item is not executed, only its resources are collected.
+                the item is not executed
             debug (bool): Whether debug mode is active or not.
 
         Raises:
-            EngineInitFailed: raised if initialization failed
+            EngineInitFailed: Raised if initialization fails
         """
-        super().__init__()
         self._queue = mp.Queue()
         if items is None:
             items = {}
         self._items = items
-        if connections is None:
-            connections = []
-        self._connections = list(map(Connection.from_dict, connections))
-        self._connections_by_source = dict()
-        self._connections_by_destination = dict()
-        node_successors = {item_name: list() for item_name in self._items}
-        for connection in self._connections:
-            if not connection.ready_to_execute():
-                notifications = " ".join(connection.notifications())
-                raise EngineInitFailed(f"Link {connection.name} is not ready for execution. {notifications}")
-            source, destination = connection.source, connection.destination
-            self._connections_by_source.setdefault(source, list()).append(connection)
-            self._connections_by_destination.setdefault(destination, list()).append(connection)
-            successors = node_successors.get(source)
-            if successors is not None:
-                successors.append(destination)
+        if execution_permits is None:
+            execution_permits = {}
+        self._execution_permits = execution_permits
+        connections = list(map(Connection.from_dict, connections))
+        project_item_loader = ProjectItemLoader()
+        self._executable_item_classes = project_item_loader.load_executable_item_classes(items_module_name)
+        required_items = required_items_for_execution(
+            self._items, connections, self._executable_item_classes, self._execution_permits
+        )
+        self._connections = make_connections(connections, required_items)
+        self._connections_by_source = {}
+        self._connections_by_destination = {}
+        self._validate_and_sort_connections()
+        self._back_injectors = dag_edges(
+            self._connections
+        )  # Mapping of a source node (item) to a list of destination nodes (items)
+        self._check_write_index()
         self._settings = AppSettings(settings if settings is not None else {})
         _set_resource_limits(self._settings, SpineEngine._resource_limit_lock)
         enable_persistent_process_creation()
         self._project_dir = project_dir
-        project_item_loader = ProjectItemLoader()
-        self._executable_item_classes = project_item_loader.load_executable_item_classes(items_module_name)
         if specifications is None:
             specifications = {}
         self._item_specifications = self._make_item_specifications(
             specifications, project_item_loader, items_module_name
         )
-        self._solid_names = {item_name: str(i) for i, item_name in enumerate(items)}
-        self._item_names = {solid_name: item_name for item_name, solid_name in self._solid_names.items()}
-        if execution_permits is None:
-            execution_permits = {}
-        self._execution_permits = {self._solid_names[name]: permits for name, permits in execution_permits.items()}
-        self._dag = make_dag(node_successors)
+        self._dag = make_dag(self._back_injectors, self._execution_permits)
         _validate_dag(self._dag)
+        self._item_names = list(self._dag)  # Names of permitted items and their neighbors
         if jumps is None:
             jumps = []
-        self._jumps = list(map(Jump.from_dict, jumps))
-        validate_jumps(self._jumps, self._dag)
+        else:
+            jumps = list(map(Jump.from_dict, jumps))
+        items_by_jump = _get_items_by_jump(jumps, self._dag)
+        self._jumps = filter_unneeded_jumps(jumps, items_by_jump, execution_permits)
+        validate_jumps(self._jumps, items_by_jump, self._dag)
         for x in self._connections + self._jumps:
             x.make_logger(self._queue)
         for x in self._jumps:
             x.set_engine(self)
-        self._back_injectors = {
-            self._solid_names[key]: [self._solid_names[x] for x in value] for key, value in node_successors.items()
-        }
         self._forth_injectors = inverted(self._back_injectors)
         self._pipeline = self._make_pipeline()
         self._state = SpineEngineState.SLEEPING
@@ -178,7 +157,63 @@ class SpineEngine:
         self._db_server_manager_queue = None
         self._thread = threading.Thread(target=self.run)
         self._event_stream = self._get_event_stream()
-        self._multiprocess_manager = mp.Manager()
+
+    def _descendants(self, name):
+        """Yields descendant item names.
+
+        Args:
+            name (str): name of the project item whose descendants to collect
+
+        Yields:
+            str: descendant name
+        """
+        for c in self._connections_by_source.get(name, ()):
+            yield c.destination
+            yield from self._descendants(c.destination)
+
+    def _check_write_index(self):
+        """Checks if write indexes are valid."""
+        conflicting_by_item = {}
+        for item_name in self._items:
+            conflicting = {}
+            descendants = self._descendants(item_name)
+            for conn in self._connections_by_source.get(item_name, ()):
+                sibling_connections = [
+                    x for x in self._connections_by_destination.get(conn.destination, []) if x != conn
+                ]
+                conflicting.update(
+                    {
+                        c.source: c.destination
+                        for c in sibling_connections
+                        if c.write_index < conn.write_index and c.source in descendants
+                    }
+                )
+            if conflicting:
+                conflicting_by_item[item_name] = conflicting
+        rows = []
+        for item_name, conflicting in conflicting_by_item.items():
+            row = []
+            for other_item_name, dest in conflicting.items():
+                row.append(f"{other_item_name}, but {other_item_name} is set to write ealier to {dest}")
+            if row:
+                rows.append(f"Item {item_name} cannot execute because it is a dependency to " + ", ".join(row))
+        msg = "\n".join(rows)
+        if msg:
+            raise EngineInitFailed(msg)
+
+    def _validate_and_sort_connections(self):
+        """Checks and sorts Connections by source and destination.
+
+        Raises:
+            EngineInitFailed: If connection is not ready
+        """
+        for connection in self._connections:
+            if not connection.ready_to_execute():
+                notifications = " ".join(connection.notifications())
+                raise EngineInitFailed(f"Link {connection.name} is not ready for execution. {notifications}")
+            source, destination = connection.source, connection.destination
+            self._connections_by_source.setdefault(source, list()).append(connection)
+            self._connections_by_destination.setdefault(destination, list()).append(connection)
 
     def _make_item_specifications(self, specifications, project_item_loader, items_module_name):
         """Instantiates item specifications.
@@ -189,7 +224,7 @@ class SpineEngine:
             items_module_name (str): name of the Python module that contains the project items
 
         Returns:
-            dict: mapping from item type to a dict that maps specification names to specification instances
+            dict: Mapping from item type to a dict that maps specification names to specification instances
         """
         specification_factories = project_item_loader.load_item_specification_factories(items_module_name)
         item_specifications = {}
@@ -208,7 +243,8 @@ class SpineEngine:
         Note that this method is called multiple times for each item:
         Once for the backward pipeline, and once for each filtered execution in the forward pipeline."""
         item_dict = self._items[item_name]
-        prompt_queue = self._prompt_queues[item_name] = mp.Queue()
+        prompt_queue = mp.Queue()
+        self._prompt_queues[id(prompt_queue)] = prompt_queue
         logger = QueueLogger(
             self._queue, item_name, prompt_queue, self._answered_prompts, silent=direction is ED.BACKWARD
         )
@@ -223,11 +259,11 @@ class SpineEngine:
 
     def get_event(self):
         """Returns the next event in the stream. Calling this after receiving the event of type "dag_exec_finished"
-        will raise StopIterationError.
-        """
+        will raise StopIterationError."""
         return next(self._event_stream)
 
     def state(self):
+        """Returns Spine Engine state."""
         return self._state
 
     def _get_event_stream(self):
@@ -241,105 +277,68 @@ class SpineEngine:
         self._thread.start()
         while True:
             msg = self._queue.get()
-            yield msg
             if msg[0] == "dag_exec_finished":
                 break
+            yield msg
+        yield msg
         self._thread.join()
 
-    def answer_prompt(self, item_name, accepted):
-        """Answers the prompt for the specified item, either accepting or rejecting it."""
-        self._prompt_queues[item_name].put(accepted)
+    def answer_prompt(self, prompter_id, answer):
+        """Answers the prompt for the specified prompter id."""
+        self._prompt_queues[prompter_id].put(answer)
 
     def wait(self):
+        """Waits until engine execution has finished."""
         if self._thread.is_alive():
             self._thread.join()
-        self._multiprocess_manager.shutdown()
 
     def run(self):
-        """Runs this engine."""
+        """Starts db server manager the engine."""
         with db_server_manager() as self._db_server_manager_queue:
             self._do_run()
 
     def _do_run(self):
+        """Runs this engine."""
         self._state = SpineEngineState.RUNNING
-        run_config = {
-            "loggers": {"console": {"config": {"log_level": "CRITICAL"}}},
-            "execution": {"multithread": {"config": {}}},
-        }
-        for event in execute_pipeline_iterator(self._pipeline, run_config=run_config):
+        for event in execute_pipeline_iterator(self._pipeline):
             self._process_event(event)
         if self._state == SpineEngineState.RUNNING:
             self._state = SpineEngineState.COMPLETED
         self._queue.put(("dag_exec_finished", str(self._state)))
 
     def _process_event(self, event):
-        """
-        Processes events from a pipeline.
+        """Processes events from a pipeline.
 
         Args:
-            event (DagsterEvent): an event
+            event (JumpsterEvent): an event
         """
-        if event.event_type == DagsterEventType.STEP_START:
-            direction, _, solid_name = event.solid_name.partition("_")
-            item_name = self._item_names[solid_name]
-            self._queue.put(('exec_started', {"item_name": item_name, "direction": direction}))
-        elif event.event_type == DagsterEventType.STEP_FAILURE and self._state != SpineEngineState.USER_STOPPED:
-            direction, _, solid_name = event.solid_name.partition("_")
-            item_name = self._item_names[solid_name]
+        if event.event_type == JumpsterEventType.STEP_START:
+            self._queue.put(("exec_started", {"item_name": event.item_name, "direction": event.direction}))
+        elif event.event_type == JumpsterEventType.STEP_FAILURE and self._state != SpineEngineState.USER_STOPPED:
             self._state = SpineEngineState.FAILED
             self._queue.put(
                 (
-                    'exec_finished',
+                    "exec_finished",
                     {
-                        "item_name": item_name,
-                        "direction": direction,
-                        "state": str(self._state),
+                        "item_name": event.item_name,
+                        "direction": event.direction,
                         "item_state": ItemExecutionFinishState.FAILURE,
                     },
                 )
             )
             if self._debug:
-                error = event.event_specific_data.error
+                error = event.error
                 print("Traceback (most recent call last):")
                 print("".join(error.stack + [error.message]))
                 print("(reported by SpineEngine in debug mode)")
-        elif event.event_type == DagsterEventType.STEP_SUCCESS:
-            # Notify Toolbox here when BACKWARD execution has finished
-            direction, _, solid_name = event.solid_name.partition("_")
-            if direction != "BACKWARD":
-                return
-            item_name = self._item_names[solid_name]
-            if not self._execution_permits[solid_name]:
-                item_finish_state = ItemExecutionFinishState.EXCLUDED
-            else:
-                item_finish_state = ItemExecutionFinishState.SUCCESS
+        elif event.event_type == JumpsterEventType.STEP_FINISH:
             self._queue.put(
                 (
-                    'exec_finished',
+                    "exec_finished",
                     {
-                        "item_name": item_name,
-                        "direction": direction,
-                        "state": str(self._state),
-                        "item_state": item_finish_state,
-                    },
-                )
-            )
-        elif event.event_type == DagsterEventType.ASSET_MATERIALIZATION:
-            # Notify Toolbox here when FORWARD execution has finished
-            direction, _, solid_name = event.solid_name.partition("_")
-            if direction != "FORWARD":
-                return
-            item_name = self._item_names[solid_name]
-            state_value = event.asset_key.path[0]
-            item_finish_state = ItemExecutionFinishState[state_value]
-            self._queue.put(
-                (
-                    'exec_finished',
-                    {
-                        "item_name": item_name,
-                        "direction": direction,
-                        "state": str(self._state),
-                        "item_state": item_finish_state,
+                        "item_name": event.item_name,
+                        "direction": event.direction,
+                        "item_state": event.item_finish_state,
                     },
                 )
             )
@@ -353,84 +352,68 @@ class SpineEngine:
         self._queue.put(("dag_exec_finished", str(self._state)))
 
     def _stop_item(self, item):
+        """Stops given project item."""
         item.stop_execution()
         self._queue.put(
             (
-                'exec_finished',
+                "exec_finished",
                 {
                     "item_name": item.name,
-                    "direction": str(ED.FORWARD),
-                    "state": str(self._state),
+                    "direction": ED.FORWARD,
                     "item_state": ItemExecutionFinishState.STOPPED,
                 },
             )
         )
 
     def _make_pipeline(self):
-        """
-        Returns a _JumpPipelineDefinition for executing this engine.
+        """Returns a PipelineDefinition for executing this engine.
 
         Returns:
-            _JumpPipelineDefinition
+            PipelineDefinition
         """
         solid_defs = [
             make_solid_def(item_name)
-            for item_name in self._items
+            for item_name in self._item_names
             for make_solid_def in (self._make_forward_solid_def, self._make_backward_solid_def)
         ]
-        dependencies = self._make_dependencies()
-        mode_defs = [
-            ModeDefinition(
-                executor_defs=default_executors + [multithread_executor],
-                resource_defs={"io_manager": shared_memory_io_manager},
-            )
-        ]
         self._complete_jumps()
-        return _JumpPipelineDefinition(
-            name="pipeline", solid_defs=solid_defs, dependencies=dependencies, mode_defs=mode_defs, jumps=self._jumps
-        )
+        return PipelineDefinition(solid_defs=solid_defs, jumps=self._jumps)
 
     def _complete_jumps(self):
-        """Returns a dictionary mapping Jump objects to a set of solid names corresponding to items within the loop.
-
-        Returns:
-            dict
-        """
+        """Updates jumps with item and corresponding solid information."""
         for jump in self._jumps:
             src, dst = jump.source, jump.destination
             jump.item_names = {dst, src}
             for path in nx.all_simple_paths(self._dag, dst, src):
                 jump.item_names.update(path)
-            jump.solid_names = {f"{ED.FORWARD}_{self._solid_names[n]}" for n in jump.item_names}
-            jump.source_solid = f"{ED.FORWARD}_{self._solid_names[src]}"
-            jump.destination_solid = f"{ED.BACKWARD}_{self._solid_names[dst]}"
 
     def _make_backward_solid_def(self, item_name):
         """Returns a SolidDefinition for executing the given item in the backward sweep.
 
         Args:
             item_name (str): The project item that gets executed by the solid.
+
+        Returns:
+            SolidDefinition: solid's definition
         """
 
-        def compute_fn(context, inputs):
+        def compute_fn(inputs):
             if self.state() == SpineEngineState.USER_STOPPED:
-                context.log.error(f"compute_fn() FAILURE with item: {item_name} stopped by the user")
                 raise Failure()
-            context.log.info(f"Item Name: {item_name}")
             item = self.make_item(item_name, ED.BACKWARD)
             resources = item.output_resources(ED.BACKWARD)
             for r in resources:
                 r.metadata["db_server_manager_queue"] = self._db_server_manager_queue
-            yield Output(value=resources, output_name=f"{ED.BACKWARD}_output")
+            yield Output(value=resources)
+            yield Finalization(
+                item_finish_state=(
+                    ItemExecutionFinishState.SUCCESS
+                    if self._execution_permits[item_name]
+                    else ItemExecutionFinishState.EXCLUDED
+                )
+            )
 
-        input_defs = []
-        output_defs = [OutputDefinition(name=f"{ED.BACKWARD}_output")]
-        return SolidDefinition(
-            name=f"{ED.BACKWARD}_{self._solid_names[item_name]}",
-            input_defs=input_defs,
-            compute_fn=compute_fn,
-            output_defs=output_defs,
-        )
+        return SolidDefinition(item_name=item_name, direction=ED.BACKWARD, input_defs=[], compute_fn=compute_fn)
 
     def _make_forward_solid_def(self, item_name):
         """Returns a SolidDefinition for executing the given item.
@@ -442,46 +425,34 @@ class SpineEngine:
             SolidDefinition
         """
 
-        def compute_fn(context, inputs):
+        def compute_fn(inputs):
             if self.state() == SpineEngineState.USER_STOPPED:
-                context.log.error(f"compute_fn() FAILURE with item: {item_name} stopped by the user")
                 raise Failure()
-            context.log.info(f"Item Name: {item_name}")
             for conn in self._connections_by_destination.get(item_name, []):
                 conn.visit_destination()
             # Split inputs into forward and backward resources based on prefix
             forward_resource_stacks = []
             backward_resources = []
-            for name, values in inputs.items():
-                if name.startswith(f"{ED.FORWARD}"):
+            for direction, values in inputs.items():
+                if direction == ED.FORWARD:
                     forward_resource_stacks += values
-                elif name.startswith(f"{ED.BACKWARD}"):
+                elif direction == ED.BACKWARD:
                     backward_resources += values
             item_finish_state, output_resource_stacks = self._execute_item(
-                context, item_name, forward_resource_stacks, backward_resources
+                item_name, forward_resource_stacks, backward_resources
             )
-            yield AssetMaterialization(asset_key=str(item_finish_state))
             if output_resource_stacks:
-                yield Output(value=output_resource_stacks, output_name=f"{ED.FORWARD}_output")
+                yield Output(value=output_resource_stacks)
             for conn in self._connections_by_source.get(item_name, []):
                 conn.visit_source()
+            yield Finalization(item_finish_state=item_finish_state)
 
         input_defs = [
-            InputDefinition(name=f"{ED.FORWARD}_input_from_{inj}")
-            for inj in self._forth_injectors.get(self._solid_names[item_name], [])
-        ] + [
-            InputDefinition(name=f"{ED.BACKWARD}_input_from_{inj}")
-            for inj in self._back_injectors.get(self._solid_names[item_name], [])
-        ]
-        output_defs = [OutputDefinition(name=f"{ED.FORWARD}_output")]
-        return SolidDefinition(
-            name=f"{ED.FORWARD}_{self._solid_names[item_name]}",
-            input_defs=input_defs,
-            compute_fn=compute_fn,
-            output_defs=output_defs,
-        )
+            InputDefinition(item_name=inj, direction=ED.FORWARD) for inj in self._forth_injectors.get(item_name, [])
+        ] + [InputDefinition(item_name=inj, direction=ED.BACKWARD) for inj in self._back_injectors.get(item_name, [])]
+        return SolidDefinition(item_name=item_name, direction=ED.FORWARD, input_defs=input_defs, compute_fn=compute_fn)
 
-    def _execute_item(self, context, item_name, forward_resource_stacks, backward_resources):
+    def _execute_item(self, item_name, forward_resource_stacks, backward_resources):
         """Executes the given item using the given forward resource stacks and backward resources.
         Returns list of output resource stacks.
 
@@ -491,12 +462,11 @@ class SpineEngine:
         ``_execute_item_filtered``.
 
         Args:
-            context
             item_name (str)
-            forward_resource_stacks (list(tuple(ProjectItemResource))): resources comming from predecessor items -
+            forward_resource_stacks (list(tuple(ProjectItemResource))): resources coming from predecessor items -
                 one tuple of ProjectItemResource per item, where each element in the tuple corresponds to a filtered
                 execution of the item.
-            backward_resources (list(ProjectItemResource)): resources comming from successor items - just one
+            backward_resources (list(ProjectItemResource)): resources coming from successor items - just one
                 resource per item.
 
         Returns:
@@ -505,9 +475,8 @@ class SpineEngine:
         """
         item = self.make_item(item_name, ED.NONE)
         if not item.ready_to_execute(self._settings):
-            if not self._execution_permits[self._solid_names[item_name]]:
+            if not self._execution_permits[item_name]:
                 return ItemExecutionFinishState.EXCLUDED, []
-            context.log.error(f"compute_fn() FAILURE with '{item_name}', not ready for forward execution")
             return ItemExecutionFinishState.FAILURE, []
         success = [ItemExecutionFinishState.NEVER_FINISHED]
         output_resources_list = []
@@ -515,22 +484,22 @@ class SpineEngine:
         resources_iterator = self._filtered_resources_iterator(
             item_name, forward_resource_stacks, backward_resources, self._timestamp
         )
-        item_lock = self._multiprocess_manager.Lock()
-        for flt_fwd_resources, flt_bwd_resources, filter_id in resources_iterator:
-            self.resources_per_item[item_name] = (flt_fwd_resources, flt_bwd_resources)
-            item = self.make_item(item_name, ED.FORWARD)
-            item.filter_id = filter_id
-            thread = threading.Thread(
-                target=self._execute_item_filtered,
-                args=(item, flt_fwd_resources, flt_bwd_resources, output_resources_list, item_lock, success),
-            )
-            threads.append(thread)
-        for thread in threads:
-            thread.start()
-        for thread in threads:
-            thread.join()
+        with mp.Manager() as multiprocess_manager:
+            item_lock = multiprocess_manager.Lock()
+            for flt_fwd_resources, flt_bwd_resources, filter_id in resources_iterator:
+                self.resources_per_item[item_name] = (flt_fwd_resources, flt_bwd_resources)
+                item = self.make_item(item_name, ED.FORWARD)
+                item.filter_id = filter_id
+                thread = threading.Thread(
+                    target=self._execute_item_filtered,
+                    args=(item, flt_fwd_resources, flt_bwd_resources, output_resources_list, item_lock, success),
+                )
+                threads.append(thread)
+            for thread in threads:
+                thread.start()
+            for thread in threads:
+                thread.join()
         if success[0] == ItemExecutionFinishState.FAILURE:
-            context.log.error(f"compute_fn() FAILURE with {item_name}, failed to execute")
             raise Failure()
         for resources in output_resources_list:
             for connection in self._connections_by_source.get(item_name, []):
@@ -552,7 +521,7 @@ class SpineEngine:
             success (list): A list of one element, to write the outcome of the execution.
         """
         self._running_items.append(item)
-        if self._execution_permits[self._solid_names[item.name]]:
+        if self._execution_permits[item.name]:
             item_finish_state = item.execute(filtered_forward_resources, filtered_backward_resources, item_lock)
             item.finish_execution(item_finish_state)
         else:
@@ -576,10 +545,10 @@ class SpineEngine:
 
         Args:
             item_name (str)
-            forward_resource_stacks (list(tuple(ProjectItemResource))): resources comming from predecessor items -
+            forward_resource_stacks (list(tuple(ProjectItemResource))): resources coming from predecessor items -
                 one tuple of ProjectItemResource per item, where each element in the tuple corresponds to a filtered
                 execution of the item.
-            backward_resources (list(ProjectItemResource)): resources comming from successor items - just one
+            backward_resources (list(ProjectItemResource)): resources coming from successor items - just one
                 resource per item.
             timestamp (str): timestamp for the execution filter
 
@@ -681,8 +650,8 @@ class SpineEngine:
         if filters is None:
             return []
         filter_configs_list = []
-        for filter_type, names in filters.items():
-            filter_configs = [filter_config(filter_type, name) for name in names]
+        for filter_type, values in filters.items():
+            filter_configs = [filter_config(filter_type, value) for value in values]
             if not filter_configs:
                 continue
             filter_configs_list.append(filter_configs)
@@ -707,7 +676,7 @@ class SpineEngine:
             resources_from_destination = resources_by_provider.get(c.destination)
             if resources_from_destination is None:
                 continue
-            if self._execution_permits[self._solid_names[item_name]]:
+            if self._execution_permits[item_name]:
                 c.clean_up_backward_resources(resources_from_destination)
             sibling_connections = [x for x in self._connections_by_destination.get(c.destination, []) if x != c]
             resources_by_provider[c.destination] = c.convert_backward_resources(
@@ -737,33 +706,6 @@ class SpineEngine:
             resources_by_provider[c.source] = c.convert_forward_resources(resources_from_source)
         return [r for resources in resources_by_provider.values() for r in resources]
 
-    def _make_dependencies(self):
-        """
-        Returns a dictionary of dependencies according to the given dictionaries of injectors.
-
-        Returns:
-            dict: a dictionary to pass to the PipelineDefinition constructor as dependencies
-        """
-        forward_deps = {
-            f"{ED.FORWARD}_{n}": {
-                f"{ED.FORWARD}_input_from_{inj}": DependencyDefinition(f"{ED.FORWARD}_{inj}", f"{ED.FORWARD}_output")
-                for inj in injs
-            }
-            for n, injs in self._forth_injectors.items()
-        }
-        backward_deps = {
-            f"{ED.FORWARD}_{n}": {
-                f"{ED.BACKWARD}_input_from_{inj}": DependencyDefinition(f"{ED.BACKWARD}_{inj}", f"{ED.BACKWARD}_output")
-                for inj in injs
-            }
-            for n, injs in self._back_injectors.items()
-        }
-        deps = {}
-        for n in forward_deps.keys() | backward_deps.keys():
-            deps[n] = forward_deps.get(n, {})
-            deps[n].update(backward_deps.get(n, {}))
-        return deps
-
 
 def _make_filter_id(resource_filter_stack):
     """Builds filter id from resource filter stack.
@@ -818,14 +760,25 @@ def _validate_dag(dag):
         raise EngineInitFailed("DAG contains unconnected items.")
 
 
-def validate_jumps(jumps, dag):
+def filter_unneeded_jumps(jumps, items_by_jump, execution_permits):
+    """Drops jumps whose items are not going to be executed.
+
+    Args:
+        jumps (Iterable of Jump): jumps to filter
+        items_by_jump (dict): mapping from jump to list of item names
+        execution_permits (dict): mapping from item name to boolean telling if its is permitted to execute
+    """
+    return [jump for jump in jumps if all(execution_permits[item] for item in items_by_jump[jump])]
+
+
+def validate_jumps(jumps, items_by_jump, dag):
     """Raises an exception in case jumps are not valid.
 
     Args:
         jumps (list of Jump): jumps
+        items_by_jump (dict): mapping from jump to list of item names
         dag (DiGraph): jumps' DAG
     """
-    items_by_jump = _get_items_by_jump(jumps, dag)
     for jump in jumps:
         validate_single_jump(jump, jumps, dag, items_by_jump)
 
@@ -835,7 +788,7 @@ def validate_single_jump(jump, jumps, dag, items_by_jump=None):
 
     Args:
         jump (Jump): the jump to check
-        jumps (list of Jump): all jumps in dag
+        jumps (list of Jump): all jumps in DAG
         dag (DiGraph): jumps' DAG
         items_by_jump (dict, optional): mapping jumps to a set of items in between destination and source
     """
