@@ -1,5 +1,6 @@
 ######################################################################################################################
 # Copyright (C) 2017-2022 Spine project consortium
+# Copyright Spine Engine contributors
 # This file is part of Spine Engine.
 # Spine Engine is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -8,16 +9,10 @@
 # Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 ######################################################################################################################
-
-"""
-Split command line arguments.
-
-:authors: P. Savolainen (VTT)
-:date:   10.1.2018
-"""
+""" Split command line arguments. """
 
 
-def split_cmdline_args(arg_string):
+def split_cmdline_args(arg_string: str) -> list[str]:
     """
     Splits a string of command line arguments into a list of tokens.
 
@@ -26,10 +21,10 @@ def split_cmdline_args(arg_string):
     Thus, `--file="a long quoted 'file' name.txt` becomes ["--file=a long quoted 'file' name.txt"]
 
     Args:
-        arg_string (str): command line arguments as a string
+        arg_string: command line arguments as a string
 
     Returns:
-        list: a list of tokens
+        a list of tokens
     """
     tokens = list()
     current_word = ""
@@ -41,7 +36,7 @@ def split_cmdline_args(arg_string):
             quoted_context = False
         elif not character.isspace() or quoted_context:
             current_word = current_word + character
-        else:
+        elif current_word:
             tokens.append(current_word)
             current_word = ""
     if current_word:
